!code to take in radius, parameter, and sampling values
!from command line, and return relative velocity values
!sampled on a regular grid based on an A3d format 
!perturbation model
!Assumes mantle model: radius values less than 3000 km are
!interpreted as depths.
!------------------------------------------------------
program A3d_slice
!------------------------------------------------------

use module_A3d
use def_gparam
use f2kcli !command line interface
implicit none

character(len=20) :: arg
character :: par,par1
real(DP) :: rad,lat,lon,theta,phi,samp
real(DP) :: dv,mean
integer :: npar,i,parn,ifrm
integer, parameter :: unit_out=357
logical :: rmean

if(command_argument_count()/=3.and.command_argument_count()/=4) &
     stop 'usage: A3d_slice rad par samp ifrmean(optional)'


call get_command_argument(1,value=arg)
read(arg,*) rad
call get_command_argument(2,value=arg)
read(arg,*) par
call get_command_argument(3,value=arg)
read(arg,*) samp
if(command_argument_count()==4) then
   call get_command_argument(4,value=arg)
   read(arg,*) ifrm
   if (ifrm==0) then
      rmean=.false.
   else if (ifrm==1) then
      rmean=.true.
   else
      stop 'invalid argument for ifrmean (should be 0 or 1)'
   endif
else
   rmean=.false.
endif

!convert depth to radius if necessary
if (rad<3000.d0) then
   print*,'Interpreting radius value ',rad,' as depth (km).'
   rad=6371.0-rad
else if (rad>6371.0) then
   print*,'Radius ',rad,' is outside the Earth. Make sure units are km.'
   stop
endif

!read model
call init_A3d(0)

!determine par number
call get_npar(npar)
parn=0
do i=1,npar
   call get_par(i,par1)
   if(par1==par) parn=i
enddo
if(parn==0) stop 'A3d_slice: invalid par choice'

open(unit_out,file='grid.out')

!determine mean value if necessary
mean=0.0
if(rmean) then
   call get_mean(rad,parn,mean)
   print*,'mean value ',100.0*mean
endif

!output file
lon=0.0
do while (lon<360.0)
  phi=deg2rad*lon
  lat=-90.0
  do while (lat<=90.0)
     theta=deg2rad*(90.0-lat)
     call A3d(rad,theta,phi,parn,dv)
     dv=dv-mean
     write(unit_out,100) lon,lat,100.0*dv
     lat=lat+samp
  enddo
  lon=lon+samp
enddo

close(unit_out)

100 format(f12.4, f12.4, f14.6)
end program A3d_slice


